<?php

namespace EventPortal\Tests\Unit\EventDelegate\Handlers;

use EventPortal\Domain\Delegate\Models\DelegateFormData;
use EventPortal\Domain\Event\Form\Models\Form;
use EventPortal\Domain\Event\Form\Models\FormStep;
use EventPortal\Domain\Event\Form\Models\FormStepSection;
use EventPortal\Domain\Event\Form\Models\Question;
use EventPortal\Domain\Event\Models\Event;
use EventPortal\Domain\Event\Models\EventDelegate;
use EventPortal\Domain\EventDelegate\Handlers\UpdateFormData;
use EventPortal\Domain\Form\Payload;
use EventPortal\Tests\TestCase;
use Illuminate\Foundation\Testing\RefreshDatabase;
use Illuminate\Pipeline\Pipeline;

class UpdateFormDataTest extends TestCase
{
    use RefreshDatabase;

    /** @test **/
    public function can_save_delegate_form_data(): void
    {
        $event = factory(Event::class)->create();
        $form = factory(Form::class)->create(['event_id' => $event->id]);
        $step = factory(FormStep::class)->create(['form_id' => $form->id]);
        $section = factory(FormStepSection::class)->create(['form_step_id' => $step->id]);
        $question = factory(Question::class)
            ->create(['form_step_section_id' => $section->id, 'question' => 'example question']);
        $eventDelegate = factory(EventDelegate::class)->create(['event_id' => $event->id]);

        $payload = Payload::make(['example_question' => 'example response'])
            ->withContext(['eventDelegate' => $eventDelegate, 'formStep' => $step]);

        app(Pipeline::class)
            ->send($payload)
            ->through([new UpdateFormData])
            ->thenReturn();

        $this->assertDatabaseHas('delegate_form_data', [
            'delegate_id' => $question->id,
            'form_step_id' => $step->id,
            'question_id' => $question->id,
            'value' => 'example response',
        ]);
        $delegateFormData = DelegateFormData::first();
        $this->assertSameModel($eventDelegate->delegate, $delegateFormData->delegate);
        $this->assertSameModel($question, $delegateFormData->question);
        $this->assertSameModel($step, $delegateFormData->step);
    }

    /** @test **/
    public function old_delegate_form_data_is_deleted(): void
    {
        $event = factory(Event::class)->create();
        $form = factory(Form::class)->create(['event_id' => $event->id]);
        $step = factory(FormStep::class)->create(['form_id' => $form->id]);
        $section = factory(FormStepSection::class)->create(['form_step_id' => $step->id]);
        $question = factory(Question::class)
            ->create(['form_step_section_id' => $section->id, 'question' => 'example question']);
        $eventDelegate = factory(EventDelegate::class)->create(['event_id' => $event->id]);
        factory(DelegateFormData::class)
            ->create([
                'delegate_id' => $eventDelegate->delegate->id,
                'form_step_id' => $step->id,
                'question_id' => $question->id,
                'value' => 'old example response',
            ]);

        $payload = Payload::make(['example_question' => 'example response'])
            ->withContext(['eventDelegate' => $eventDelegate, 'formStep' => $step]);

        app(Pipeline::class)
            ->send($payload)
            ->through([new UpdateFormData])
            ->thenReturn();

        $this->assertDatabaseMissing('delegate_form_data', [
            'delegate_id' => $eventDelegate->delegate->id,
            'form_step_id' => $step->id,
            'question_id' => $question->id,
            'value' => 'old example response',
        ]);
        $this->assertDatabaseHas('delegate_form_data', [
            'delegate_id' => $eventDelegate->delegate->id,
            'form_step_id' => $step->id,
            'question_id' => $question->id,
            'value' => 'example response',
        ]);
        $this->assertCount(1, DelegateFormData::all());
    }

    /** @test **/
    public function old_form_data_is_deleted_for_the_right_delegate(): void
    {
        $event = factory(Event::class)->create();
        $form = factory(Form::class)->create(['event_id' => $event->id]);
        $step = factory(FormStep::class)->create(['form_id' => $form->id]);
        $section = factory(FormStepSection::class)->create(['form_step_id' => $step->id]);
        $question = factory(Question::class)
            ->create(['form_step_section_id' => $section->id, 'question' => 'example question']);
        $eventDelegateA = factory(EventDelegate::class)->create(['event_id' => $event->id]);
        $eventDelegateB = factory(EventDelegate::class)->create(['event_id' => $event->id]);
        $oldDelegateFormDataA = factory(DelegateFormData::class)->create([
            'delegate_id' => $eventDelegateA->delegate->id,
            'form_step_id' => $step->id,
            'question_id' => $question->id,
            'value' => 'old example response',
        ]);
        $oldDelegateFormDataB = factory(DelegateFormData::class)->create([
            'delegate_id' => $eventDelegateA->delegate->id,
            'form_step_id' => $step->id,
            'question_id' => $question->id,
            'value' => 'old example response',
        ]);

        $payload = Payload::make(['example_question' => 'example response'])
            ->withContext(['eventDelegate' => $eventDelegateB, 'formStep' => $step]);

        app(Pipeline::class)
            ->send($payload)
            ->through([new UpdateFormData])
            ->thenReturn();

        $this->assertDatabaseMissing('delegate_form_data', [
            'id' => $oldDelegateFormDataB->id,
            'delegate_id' => $eventDelegateB->delegate->id,
            'form_step_id' => $step->id,
            'question' => $question->id,
            'value' => 'old example response',
        ]);

        $this->assertDatabaseHas('delegate_form_data', [
            'id' => $oldDelegateFormDataA->id,
            'delegate_id' => $eventDelegateA->delegate->id,
            'form_step_id' => $step->id,
            'question_id' => $question->id,
            'value' => 'old example response',
        ]);
        $this->assertDatabaseHas('delegate_form_data', [
            'delegate_id' => $eventDelegateB->delegate->id,
            'form_step_id' => $step->id,
            'question_id' => $question->id,
            'value' => 'example response',
        ]);
        $this->assertCount(3, DelegateFormData::all());
    }
}
